/*********************************************************************
* Copyright (c) 2023 Contributors to the Eclipse Foundation.
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which is available at https://www.eclipse.org/legal/epl-2.0/
*
* SPDX-License-Identifier: EPL-2.0
*
* Contributors:
*   Kentyou - initial implementation
**********************************************************************/
package org.eclipse.sensinact.gateway.northbount.sensorthings.mqtt.mappers;

import java.time.Instant;
import java.util.stream.Stream;

import org.eclipse.sensinact.core.command.GatewayThread;
import org.eclipse.sensinact.core.notification.ResourceDataNotification;
import org.eclipse.sensinact.core.twin.TimedValue;
import org.eclipse.sensinact.gateway.northbount.sensorthings.mqtt.SensorthingsMapper;
import org.eclipse.sensinact.sensorthings.sensing.dto.Observation;
import org.osgi.util.promise.Promise;

import com.fasterxml.jackson.databind.ObjectMapper;

public class ObservationsMapper extends SensorthingsMapper<Observation> {

    public ObservationsMapper(String topicFilter, ObjectMapper mapper, GatewayThread thread) {
        super(topicFilter, mapper, thread);
    }

    public Promise<Stream<Observation>> toPayload(ResourceDataNotification notification) {
        TimedValue<Object> tv = new TimedValue<Object>() {

            @Override
            public Instant getTimestamp() {
                return notification.timestamp;
            }

            @Override
            public Object getValue() {
                return notification.newValue;
            }

        };
        return wrap(DtoMapper.toObservation(notification.provider, notification.service, notification.resource, tv));
    }

    @Override
    protected Class<Observation> getPayloadType() {
        return Observation.class;
    }
}
